# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    This module copyright (C) 2013 Savoir-faire Linux
#    (<http://www.savoirfairelinux.com>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import time
from openerp.osv import orm, fields


class res_partner_category(orm.Model):
    """
    Inherits partner_category
    """
    _inherit = 'res.partner.category'
    _columns = {
        'categoryfunction_ids': fields.one2many('res.category.functions', 'category_id', 'Functions'),
    }


class res_category_functions(orm.Model):
    """
    Adds this class to link category and functions
    """
    _description = 'Categories functions'
    _name = 'res.category.functions'
    _order = 'sequence asc'
    _columns = {
        'functions_id': fields.many2one('functions', 'Functions', required='True'),
        'sequence': fields.integer('Sequence', help="Used to order"),
        'category_id': fields.many2one('res.partner.category', 'Category'),
    }


class res_partner(orm.Model):
    """
    Inherits partner and adds functions_ids : List of functions
    """
    _inherit = 'res.partner'

    def onchange_partner_function(self, cr, uid, ids, part, context=None):
        if not part:
            return {'value': None}
        partner_ids = self.pool.get('res.partner').search(cr, uid, [('id', '=', part)])
        functions_ids = self.pool.get('res.partner').browse(cr, uid, partner_ids, context=context)[0].functions_ids
        result = []
        if functions_ids:
            for line in functions_ids:
                result.append(line.id)
        dom = {'function_id': [('id', 'in', result)]}
        return {'domain': dom}

    def _get_history_lines(self, cr, uid, ids, name, arg, context=None):
        res = {}
        res_partner_pool = self.pool.get('res.partner')
        for record in self.browse(cr, uid, ids, context=context):

            if record.is_company:
                return res
            contact_ids = res_partner_pool.search(cr, uid,
                                                  [('contact_id', '=', record.id),
                                                   ('active', '=', False)],
                                                  context=context)
            res[record.id] = [x.id for x in res_partner_pool.browse(cr, uid, contact_ids)
                              if x.end_date and x.end_date <= time.strftime('%Y-%m-%d')]
        return res

    def create(self, cr, user, vals, context=None):
        context = self._basecontact_check_context(cr, user, 'create', context)
        if not vals.get('name') and vals.get('contact_id'):
            vals['name'] = self.browse(cr, user, vals['contact_id'], context=context).name
        if vals.get('end_date'):
            if vals['end_date'] and vals['end_date'] <= time.strftime('%Y-%m-%d'):
                vals['active'] = False
        if vals.get('contact_type') and vals.get('contact_type') == 'standalone':
            contact_vals = {key: vals[key] for key in vals.keys() if key != 'parent_id'}
            contact_vals['active'] = True
            vals['contact_id'] = super(res_partner, self).create(cr, user, contact_vals,
                                                                 context=context)
            self.write(cr, user, vals['contact_id'], {'lastname': vals['lastname']}, context=context)
        if 'reset_password' in context.keys() or 'install_mode' in context.keys():
            return vals['contact_id']
        res = super(res_partner, self).create(cr, user, vals, context=context)
        if vals.get('lastname'):
            self.write(cr, user, res, {'lastname': vals['lastname']}, context=context)
        return res

    def write(self, cr, user, ids, vals, context=None):
        context = self._basecontact_check_context(cr, user, 'write', context)
        if vals.get('end_date'):
            if vals['end_date'] and vals['end_date'] <= time.strftime('%Y-%m-%d'):
                vals['active'] = False
        if vals.get('contact_type') and vals.get('contact_type') == 'standalone':
            contact_vals = {key: vals[key] for key in vals.keys() if key != 'parent_id'}
            contact_vals['active'] = True
            vals['contact_id'] = super(res_partner, self).write(cr, user, contact_vals,
                                                                context=context)
        return super(res_partner, self).write(cr, user, ids, vals, context=context)

    def _compute_name_custom(self, cursor, uid, ids, fname, arg, context=None):
        res = {}
        partners = self.read(cursor, uid, ids,
                             ['firstname', 'lastname'], context=context)
        for rec in partners:
            names = (rec['firstname'], rec['lastname'])
            fullname = " ".join([s for s in names if s])
            res[rec['id']] = fullname
        return res

    _columns = {
        'functions_ids': fields.many2many('functions', 'function_partner_rel', 'partner_id',
                                          'function_id', 'Functions'),
        'delivered_country': fields.many2one('res.country', 'Country Passport',
                                             help='Country for which a passport is delivered'),
        'passport_number': fields.char('Passport Number', size=256,
                                       help="Passport number."),
        'expiration_date': fields.date('Expiration date'),
        'start_date': fields.date('Start date'),
        'end_date': fields.date('End date'),
        'naming': fields.char('Naming', size=256, help="Naming."),
        'function_id': fields.many2one('functions', 'Position Occupied'),
        'other_contact_history_ids': fields.function(_get_history_lines,
                                                     relation="res.partner",
                                                     method=True,
                                                     type="one2many"),
        'name': fields.function(_compute_name_custom, string="Name",
                                type="char",
                                store=True,
                                select=True,
                                readonly=True),
    }

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
