# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    This module copyright (C) 2013 Savoir-faire Linux
#    (<http://www.savoirfairelinux.com>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.tests.common import TransactionCase
from openerp.osv.orm import browse_record


class Base_Test_institution(TransactionCase):
    """
    Simple test creating a institution
    This is a base class for institution test cases.
    Inherit from this and setup values.
    """

    def setUp(self, vals={}):
        """
        Setting up institution.
        """
        # Default test values
        self.vals = {'name': 'This is a test institution name',
                     'sequence': '1',
                     'institutionfunction_ids': '[]',
                     }
        super(Base_Test_institution, self).setUp()
        self.vals = dict(self.vals.items() + vals.items())
        # Create the institution object; we will be testing this, so store in self
        institution_institution = self.registry('res.institution')
        self.institution_id = institution_institution.create(self.cr, self.uid, self.vals, context=None)

    def test_institution(self):
        """
        Checking the institution creation.
        """
        institution_institution = self.registry('res.institution')
        institution_obj = institution_institution.browse(self.cr, self.uid, self.institution_id, context=None)
        for field in self.vals:
            val = institution_obj[field]
            if type(val) == browse_record:
                self.assertEquals(self.vals[field], val.id,
                                  "IDs for %s don't match: (%i != %i)" %
                                  (field, self.vals[field], val.id))
            else:
                self.assertEquals(str(self.vals[field]), str(val),
                                  "Values for %s don't match: (%s != %s)" %
                                  (field, str(self.vals[field]), str(val)))


class Test_institution_bad(Base_Test_institution):
    """
    Simple test creating a institution, test against bad values
    """
    def setUp(self):
        """
        Setting up institution, then changing the values to test against.
        """
        super(Test_institution_bad, self).setUp()
        # Change vals to something wrong
        self.vals = {'name': 'This is the wrong institution name',
                     'sequence': '0',
                     }

    def test_institution(self):
        """
        Checking the institution creation, assertions should all be false.
        """
        institution_institution = self.registry('res.institution')
        institution_obj = institution_institution.browse(self.cr, self.uid, self.institution_id, context=None)
        for field in self.vals:
            val = institution_obj[field]
            if type(val) == browse_record:
                self.assertNotEqual(self.vals[field], val.id,
                                    "IDs for %s don't match: (%i != %i)" %
                                    (field, self.vals[field], val.id))
            else:
                self.assertNotEqual(str(self.vals[field]), str(val),
                                    "Values for %s don't match: (%s != %s)" %
                                    (field, str(self.vals[field]), str(val)))


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
